import sys
import os
import string
import time
# modify import path instead of adding a __init__.py as I want to keep 
# a the Sulley install as unmodified as possible to facilitate easy updating 
# if there are changes to it. 
# Props to Pedram Amini and Aaron Portnoy for creating such a great framework
sys.path.append(''.join([os.getcwd(), '/sulley']))

from random import Random

from protocol_logic.sip_utilities import SIPCrashDetector
from protocol_logic.sip_utilities import SIPInviteCanceler
from misc.utilities import Logger

from fuzzer_parents import AbstractFuzzer
from fuzzer_parents import AbstractSIPFuzzer
from fuzzer_parents import AbstractSIPInviteFuzzer

from socket import *        
from sulley import *

################################################################################
class Callable:
    def __init__(self, anycallable):
        '''
        Wrapper class so I can have unbound class methods. Apparently python doesn't
        allow these by default. This code/idiom came from some standard example on the
        Interwebs
        '''
        self.__call__ = anycallable

class SIPInviteStructureFuzzer(AbstractSIPInviteFuzzer):
    '''
    Fuzz the structure of an INVITE request e.g repeats, line folding etc
    '''
    def fuzz(self):
        self.sess.add_target(self.target)
        self.sess.connect(s_get("INVITE_STRUCTURE"), callback=self.generate_unique_attributes)
        self.sess.fuzz()

    def info(self=None):
        h = ["Name: SIPInviteStructureFuzzer\n",
             "Protocol: SIP\n",
             "Success Factor: Low\n",
             "Fuzzes the structure of a SIP request by repeating blocks, fuzzing delimiters and ",
             "generally altering how a SIP request is structured",
             ]
        return ''.join(h)

    info = Callable(info)
        
class SIPInviteRequestLineFuzzer(AbstractSIPInviteFuzzer):
    def fuzz(self):
        self.sess.add_target(self.target)
        self.sess.connect(s_get("INVITE_REQUEST_LINE"), callback=self.generate_unique_attributes)
        self.sess.fuzz()

    def info(self=None):
        h = ["Name: SIPInviteRequestLineFuzzer\n",
             "Protocol: SIP\n",
             "Success Factor: Low\n",
             "Extensively tests the first line of an INVITE request by including all valid parts ",
             "specified in SIP RFC 3375"
             ]
        return ''.join(h)

    info = Callable(info)
        
class SIPInviteCommonFuzzer(AbstractSIPInviteFuzzer):    
    def fuzz(self):
        self.sess.add_target(self.target)
        self.sess.connect(s_get("INVITE_COMMON"), callback=self.generate_unique_attributes)
        self.sess.fuzz()

    def info(self=None):        
        h = ["Name: SIPInviteCommonFuzzer\n",
             "Protocol: SIP\n",
             "Success Factor: High\n",
             "Fuzzes the headers commonly found and most likely to be processed in a SIP INVITE request\n"
             ]
        return ''.join(h)
    
    info = Callable(info)    

class SIPInviteOtherFuzzer(AbstractSIPInviteFuzzer):
    def fuzz(self):
        self.sess.add_target(self.target)
        self.sess.connect(s_get("INVITE_OTHER"), callback=self.generate_unique_attributes)
        self.sess.fuzz()

    def info(self=None):
        h = ["Name: SIPInviteOtherFuzzer\n",
             "Protocol: SIP\n",
             "Success Factor: Low\n",
             "Tests all other headers specified as part of an INVITE besides those found in the ",
             "SIPInviteCommonFuzzer. Many of these are seemingly unparsed and ignored by a lot of devices.\n"
             ]
        return ''.join(h)

    info = Callable(info)    

class SDPFuzzer(AbstractSIPInviteFuzzer):
    '''
    Extends the Abstract INVITE fuzzer because it requires the INVITE
    cancelling functionality. Fuzzes the SDP content of an INVITE.
    '''
    
    def fuzz(self):        
        self.sess.add_target(self.target)
        self.sess.connect(s_get("SDP"), callback=self.generate_unique_attributes)
        self.sess.fuzz()

    def info(self=None):
        h = ["Name: SDPFuzzer\n",
             "Protocol: SDP\n",
             "Success Factor: High\n",
             "Fuzzes the SDP protocol as part of a SIP INVITE\n"
             ]
        return ''.join(h)
            
    info = Callable(info)

class SIPDumbACKFuzzer(AbstractSIPFuzzer):
    '''
    A dumb ACK fuzzer that doesn't wait for any kind of responses or what not.    
    '''

    def fuzz(self):        
        self.sess.add_target(self.target)
        self.sess.connect(s_get("ACK"))
        self.sess.fuzz()
        
    def info(self=None):
        h = ["Name: SIPDumbACKFuzzer\n",
             "Protocol: SIP\n",
             "Success Factor: Unknown\n",
             "A dumb ACK fuzzer with no transaction state awareness. \n",
             ]
        return ''.join(h)
            
    info = Callable(info)

class SIPDumbCANCELFuzzer(AbstractSIPFuzzer):
    '''
    A dumb CANCEL fuzzer that doesn't wait for any kind of responses or what not.    
    '''

    def fuzz(self):        
        self.sess.add_target(self.target)
        self.sess.connect(s_get("CANCEL"))
        self.sess.fuzz()
        
    def info(self=None):
        h = ["Name: SIPDumbCANCELFuzzer\n",
             "Protocol: SIP\n",
             "Success Factor: Unknown\n",
             "A dumb CANCEL request fuzzer with no transaction state awareness\n",
             ]
        return ''.join(h)
            
    info = Callable(info) 
    
'''
class SDPEncodedFuzzer(AbstractSIPInviteFuzzer):
    
    def fuzz(self):        
        self.sess.add_target(self.target)
        self.sess.connect(s_get("SDP_ENCODED"), callback=self.generate_unique_attributes)
        self.sess.fuzz()

class OptionsFuzzer(AbstractSIPFuzzer):
    def fuzz(self):
        self.sess.add_target(self.target)
        self.sess.connect(s_get("OPTIONS"), callback=self.generate_unique_attributes)
        self.sess.fuzz()
'''
        
################################################################################
